#!/bin/bash

usage() {
  echo "Usage: $0 -a <MOCKED_ASSET_DATA_PATH> -c <CURRENT_MONTH_DATA_PATH> -l <LAST_MONTH_DATA_PATH> -v <CACHE_VERSION>" 1>&2
  echo "-a ASSET_DATA_PATH "
  echo "-c CURRENT_MONTH_ICM_DATA_PATH "
  echo "-l LAST_MONTH_ICM_DATA_PATH "
  echo "-v CACHE VERSION "
}


# Prints a formatted error to stderr and exits unsuccessfully.
# $1 - error text, $2 - error code (default 1)
fail_with_error() {
  if [ -z $2 ]; then
    ret=1
  else
    ret=$2
  fi
  echo -e "$1" 1>&2
  echo ""
  usage
  exit $ret
}

while getopts ":a:b:c:l:v:i:u:p:" FLAG; do
  case $FLAG in
    a)
      ASSET_DATA_PATH=$OPTARG
      ;;
    b)
      ACCOUNT_DATA_PATH=$OPTARG
      ;;
    c)
      CURRENT_MONTH=$OPTARG
      ;;
    l)
      LAST_MONTH=$OPTARG
      ;;
    v)
      CACHE_VERSION=$OPTARG
      ;;
    i)
      ICM_DATA_PATH=$OPTARG
      ;;
    u)
      CUSTOMER_ID=$OPTARG
      ;;
    p)
      PAYER_ACCOUNT_ID=$OPTARG
      ;;
  esac
done

if [ $# -lt 1 ]; then
    echo "Your command line contains $# arguments"
    exit 1
fi


if [ -z $ASSET_DATA_PATH ]; then
  fail_with_error "You must provide Mocked Asset Data S3 location "
fi

if [ -z $ACCOUNT_DATA_PATH ]; then
  fail_with_error "You must provide Mocked ACCOUNT Data S3 location "
fi

if [ -z $CURRENT_MONTH ]; then
  fail_with_error "You must provide Mocked ICM Data path for Current month usage"
fi

if [ -z $LAST_MONTH ]; then
  fail_with_error "You must provide Mocked ICM Data path for last month usage"
fi

if [ -z $CACHE_VERSION ]; then
  fail_with_error "You must provide Cache version"
fi

if [ -z CUSTOMER_ID ]; then
  fail_with_error "You must provide Customer ID"
fi

if [ -z $PAYER_ACCOUNT_ID ]; then
  fail_with_error "You must provide PayerAccount ID"
fi

json_data=$(cat <<EOF
{
  "customerId": $CUSTOMER_ID,
  "cacheVersion": "$CACHE_VERSION",
  "orgId": "3229815411077",
  "cloudType": "aws",
  "s3Path": {
    "AwsInstanceReservation": "$ASSET_DATA_PATH",
    "AwsCostManagementSavingsPlan": "s3://cht-analysis-20-staging/data-export/production/db-attribute-export/$CUSTOMER_ID/$CACHE_VERSION/AwsCostManagementSavingsPlan_attribute_export_1.csv.gz",
    "AwsAccount": "$ACCOUNT_DATA_PATH",
    "XForm": {
      "$PAYER_ACCOUNT_ID": {
        "CurrentMonthUsage": "$CURRENT_MONTH",
        "LastMonthUsage": "$LAST_MONTH"
      }
    }
  }
}
EOF
)

echo "$json_data" > recos.json

json_data_source=$(cat <<EOF
{
  "lastAnalysisStartedOn": `date +'%s'`,
  "sourceData": [
    {
      "lastUpdatedOn": `date +'%s'`,
      "type": "asset_cache",
      "month": null,
      "identifier": null
    },
    {
      "lastUpdatedOn": `date +'%s'`,
      "type": "icm_dataset",
      "month": "`date +'%Y-%m'`",
      "identifier": null
    },
    {
      "lastUpdatedOn": `date +'%s'`,
      "type": "icm_dataset",
      "month": "`date +'%Y-%m'`",
      "identifier": null
    }
  ]
}
EOF
)

echo "$json_data_source" > source_data.json

json_data_preference=$(cat <<EOF
{
  "initialVersionCreatedBy": "crn:1:user/139308",
  "lastUpdatedBy": "crn:1:user/139308",
  "initialVersionCreatedOn": 1642084766085,
  "lastUpdatedOn": 1642084766085,
  "revision": "latest",
  "version": 1,
  "allRevisions": null,
  "customer": "crn:41325:tenant/41325",
  "id": "crn:41325:inline-cost-management/AWS-preference:3229815415545",
  "accountId": "crn:41325:aws-account/3229815415545",
  "customerId": "41325",
  "cloudType": "AWS",
  "expectedUsageTrend": "INCREASING",
  "expectedWorkloadStability": "STABLE",
  "riskAffinity": "BALANCED_PORTFOLIO",
  "prePayBudget": {
    "startMonth": "2022-01",
    "endMonth": "2022-12",
    "budgetAllocated": 20000,
    "budgetSpent": 0,
    "budgetSpentHistory": {}
  },
  "dryRun": true,
  "onboarded": true,
  "customerIdPayerAccId": "41325|3229815415545"
}
EOF
)

echo "$json_data_preference" > preference.json
