# frozen_string_literal: true

require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

ICM_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

# module for icm
module ChtApitestIcm
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module IcmHelpers
    # insertion point for shortcuts

    def create_savings_automator_account_preference(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('create_savings_automator_account_preference')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { input: { accountId: args[:accountId], cloudType: args[:cloudType], expectedUsageTrend: args[:expectedUsageTrend], expectedWorkloadStability: args[:expectedWorkloadStability], prePayBudget: { budgetAllocated: args[:budgetAllocated], endMonth: args[:endMonth], startMonth: args[:startMonth] }, riskAffinity: args[:riskAffinity] } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def update_savings_automator_account_preference(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('update_savings_automator_account_preference')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { input: { dryRun: args[:dryRun], expectedUsageTrend: args[:expectedUsageTrend], expectedWorkloadStability: args[:expectedWorkloadStability], id: args[:id], prePayBudgetAllocated: args[:prePayBudgetAllocated], prePayBudgetSpend: args[:prePayBudgetSpend], riskAffinity: args[:riskAffinity] } }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

=begin
    def update_savings_automator_action(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('update_savings_automator_action')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = args[:body] || format_request(sym, args)
      variables = {"input"=>{"id"=>args[:id], "status"=>args[:status]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end


    def enable_xform_backfill(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('enable_xform_backfill')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = args[:body] || format_request(sym, args)
      variables = {"input"=>{"id"=>args[:id]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end
=end

    def node(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('node')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = args[:body] || format_request(sym, args)
      variables = {"id"=>args[:id]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def savings_automator_account_preferences(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('savings_automator_account_preferences')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { after: args[:after], cloud: args[:cloud], first: args[:first] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def savings_automator_recommended_actions(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('savings_automator_recommended_actions')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { after: args[:after], filterRules: args[:filterRules], first: args[:first] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def savings_automator_data_provenance(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('savings_automator_data_provenance')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { filterRules: args[:filterRules] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def savings_automator_summaries(args = {})
      args[:service] = IcmService.new
      sym = first_no_cap(camelize('savings_automator_summaries')).to_sym
      headers = apitest_headers(
        args.merge(with_auth: true,
        auth_service: LocalAuthService.new(via_public: via_public)))
      data = format_request(sym, args)
      variables = { filterRules: args[:filterRules] }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    # end of shortcut insertion
  end

  # class for your service
  class IcmService < ChtNgService
    include RSpec::Matchers

    ICM_DEFAULTS = {
      name: 'Icm',
      namespace: 'icm',
      default_namespace: 'icm',
      service_name: 'icm',
      component_name: 'api',
      pr_parent: 'icm',
      formatters: ICM_FMTRS['formatters'],
      checkers: ICM_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(ICM_DEFAULTS))
    end
  end

  # class for api-proxy support of icm service
  class ApiProxyIcm < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxyIcm',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: ICM_FMTRS['formatters'],
      checkers: ICM_FMTRS['checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
      @via_public = true
    end
  end
end
