require 'spec_helper'

module IcmTest
  describe 'Icm userPreferences' do
    include_context 'request metadata'

    before(:all) do
      $accountid
      $customerId
      $test_env = YAML::load_file(File.expand_path("../lib/support/yml/test_environment.yml", __dir__))
      $test_env.each do |key1, val1|
        if ( key1.to_s == ENV['TEST_ENV'] )
          $test_env[:"#{key1}"].each do | key, val |
             if (key.to_s == 'payeraccount')
               $accountid = val.to_s
             elsif (key.to_s == 'customerId')
               $customerId = val.to_s
             end
          end
        end
      end
    end

    it 'Create SavingsAutomator Account Preference' do
      wrap_request do
        variables = {
            'input': {
                'accountId': 'crn:'+ $customerId + ':aws-account/'+ $accountid,
                'cloudType': 'AWS',
                'expectedUsageTrend': 'DECREASING',
                'expectedWorkloadStability': 'STABLE',
                'prePayBudget': {
                    'budgetAllocated': 10000.0,
                    'endMonth': '2023-06',
                    'startMonth': '2022-07'
                },
                'riskAffinity': 'BALANCED_PORTFOLIO'
            }
        }
        graphql url(service: service),
                format_request(
                    :createSavingsAutomatorAccountPreference,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:createSavingsAutomatorAccountPreference, service: service)
        expect_json 'data.createSavingsAutomatorAccountPreference.accountPreference',
                    accountId: 'crn:'+ $customerId + ':aws-account/'+ $accountid,
                    cloudType: "AWS",
                    expectedUsageTrend: "DECREASING",
                    expectedWorkloadStability: "STABLE",
                    riskAffinity: "BALANCED_PORTFOLIO"
                    # Validate the default option for dry run should be true
                    # dryRun: false
        #expect(json_body[:data][:createICMAccountPreference][:accountPreference][:prePayBudget][:budgetAvailable]).to eq 10000.0
      end
    end

    it 'Update Savings Automator Account Preference' do
      wrap_request do
        variables = {
            'input': {
                #'dryRun': true,
                'expectedUsageTrend': 'INCREASING',
                'expectedWorkloadStability': 'STABLE',
                'id': 'crn:' + $customerId +':inline-cost-management/AWS-preference:' + $accountid,
                'prePayBudgetSpend': 0.10,
                'riskAffinity': 'BALANCED_PORTFOLIO'
            }
        }
        graphql url(service: service),
                format_request(
                    :updateSavingsAutomatorAccountPreference,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:updateSavingsAutomatorAccountPreference, service: service)
        expect_json 'data.updateSavingsAutomatorAccountPreference.accountPreference',
                    accountId: 'crn:'+ $customerId + ':aws-account/'+ $accountid,
                    cloudType: "AWS",
                    expectedUsageTrend: "INCREASING",
                    expectedWorkloadStability: "STABLE",
                    riskAffinity: "BALANCED_PORTFOLIO"
        #expect(json_body[:data][:updateICMAccountPreference][:accountPreference][:prePayBudget][:budgetSpent]).to eq 0.10
      end
    end

    it 'Savings Automator Account Preference' do
      wrap_request do
        variables = nil
        graphql url(service: service),
                format_request(
                    :savingsAutomatorAccountPreferences,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorAccountPreferences, service: service)
      end
    end

    it 'Savings Automator Created Recommended Actions' do
      wrap_request do
        variables = {
            #'after': 'after',
            'filterRules': [ {
                field: "accountId",
                filterType: "EQ",
                filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
                             },
                             {
                                 "field": "status",
                                 "filterType": "EQ",
                                 "filterValues": [
                                     "CREATED"
                                 ]
                             }
            ],

            #'first': 'first'
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorRecommendedActions,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorRecommendedActions, service: service)
      end
    end

    it 'Savings Automator Completed Recommended Actions' do
      wrap_request do
        variables = {
            #'after': 'after',
            'filterRules': [ {
                field: "accountId",
                filterType: "EQ",
                filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
                             },
                             {
                                 "field": "status",
                                 "filterType": "EQ",
                                 "filterValues": [
                                     "COMPLETED"
                                 ]
                             }
            ],

            #'first': 'first'
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorRecommendedActions,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorRecommendedActions, service: service)
      end
    end

    it 'Savings Automator Upcoming Recommended Actions' do
      wrap_request do
        variables = {
            #'after': 'after',
            'filterRules': [ {
                                 field: "accountId",
                                 filterType: "EQ",
                                 filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
                             },
                             {
                                 "field": "status",
                                 "filterType": "EQ",
                                 "filterValues": [
                                     "UPCOMING"
                                 ]
                             }
            ],

            #'first': 'first'
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorRecommendedActions,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorRecommendedActions, service: service)
      end
    end

    it 'Savings Automator Failed Recommended Actions' do
      wrap_request do
        variables = {
            #'after': 'after',
            'filterRules': [ {
                                 field: "accountId",
                                 filterType: "EQ",
                                 filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
                             },
                             {
                                 "field": "status",
                                 "filterType": "EQ",
                                 "filterValues": [
                                     "FAILED"
                                 ]
                             }
            ],

            #'first': 'first'
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorRecommendedActions,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorRecommendedActions, service: service)
      end
    end

    it 'Savings Automator DataProvenance' do
      wrap_request do
        variables = {
            'filterRules': [
                field: "accountId",
                filterType: "EQ",
                filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
            ]
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorDataProvenance,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorDataProvenance, service: service)
      end
    end

    it 'Savings Automator Summaries' do
      wrap_request do
        variables = {
            'filterRules': [
                field: "accountId",
                filterType: "EQ",
                filterValues: ['crn:'+ $customerId + ':aws-account/'+ $accountid]
            ]
        }
        graphql url(service: service),
                format_request(
                    :savingsAutomatorSummaries,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:savingsAutomatorSummaries, service: service)
      end
    end

=begin
    it 'Update Saving Automator Action' do
      wrap_request do
        variables = {
            "input": {
                "id": "ID",
                "status": "SavingsAutomatorActionStatus"
            }
        }
        graphql url(service: service),
                format_request(
                    :updateSavingsAutomatorAction,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:updateSavingsAutomatorAction, service: service)
      end
    end
=end

=begin

    # This test needs to be improved to get the input from other Endpoint output
    it 'node spec' do
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                    :node,
                    service: service,
                    crn: 'crn:2017:inline-cost-management/AWS-action:25c2bc0d-ec49-45d3-acad-d1eb0fbb52b5',
                    type: 'ICMRecommendedAction',
                    fields: 'strategy estimatedEffectiveSavingsRate estimatedCost estimatedLifetimeSavings id accountId status estimatedMonthlySavings date'
                ),
                variables, headers
        expect_success_graphql
        check(:node, service: service)
        # expect_json 'data.node',
        #             strategy: "SMART_BOOST",
        #             accountId: "crn:2017:aws-account/3367254377911"
      end
    end
=end

  end
end