require 'spec_helper'

module IcmTest
  describe 'Icm E2E Tests', e2e_tests: true do
    include_context 'request metadata'

    before(:all) do
      $merge_ids = {}
      $boost_ids = {}
      $reuse_ids = {}
      $e2e_inp = YAML::load_file(File.expand_path("../lib/support/yml/e2e_inp.yml", __dir__))
      $today = Date.today
    end

    it 'ICM Recommended Actions' do
      wrap_request do
        variables = {
            'filterRules': [
                {
                field: "accountId",
                filterType: "EQ",
                filterValues: ["crn:41325:aws-account/3229815415545"]
                }
            ]
        }
        graphql url(service: service),
                format_request(
                    :icmRecommendedActions,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:icmRecommendedActions, service: service)

        k = json_body[:data][:icmRecommendedActions][:edges]
        $merge_ids = {}
        $boost_ids = {}
        $reuse_ids = {}

        k.each do |item|
          #if (item[:node][:strategy] == "SMART_MERGE" and item[:node][:date].include? $today.strftime())
          #2022-01-12
          if (item[:node][:strategy] == "SMART_MERGE" and item[:node][:date].include? $today.strftime())
            $merge_ids[item[:node][:id]] = item[:node][:date]
          elsif ( item[:node][:strategy] == "SMART_REUSE" and item[:node][:date].include? $today.strftime())
            $reuse_ids[item[:node][:id]] = item[:node][:date]
          elsif (item[:node][:strategy] == "SMART_BOOST" and item[:node][:date].include? $today.strftime())
            $boost_ids[item[:node][:id]] = item[:node][:date]
          else
            next
          end
        end
      end
    end

    it 'ICM Summaries' do
      wrap_request do
        variables = {
            'filterRules': [
                field: "accountId",
                filterType: "EQ",
                filterValues: ["crn:41325:aws-account/3229815415545"]
            ]
        }
        graphql url(service: service),
                format_request(
                    :icmSummaries,
                    service: service
                ),
                variables, headers
        expect_success_graphql
        check(:icmSummaries, service: service)
        # expect_json 'data.icmSummaries.prePayBudgetSummary',
        #             total: $e2e_inp[:icmSummaries][:prePayBudgetSummary]['total'],
        #             change: $e2e_inp[:icmSummaries][:prePayBudgetSummary]['change'],
        #             budgetAvailable: $e2e_inp[:icmSummaries][:prePayBudgetSummary]['budgetAvailable'],
        #             budgetSpent: $e2e_inp[:icmSummaries][:prePayBudgetSummary]['budgetSpent']
        # expect_json 'data.icmSummaries.effectiveSavingsRateSummary',
        #             total: $e2e_inp[:icmSummaries][:effectiveSavingsRateSummary]['total'],
        #             change: $e2e_inp[:icmSummaries][:effectiveSavingsRateSummary]['change']
        # expect_json 'data.icmSummaries.savingsSummary',
        #             total: $e2e_inp[:icmSummaries][:savingsSummary]['total'],
        #             change: $e2e_inp[:icmSummaries][:savingsSummary]['change']
      end
    end

    it "SmartMerge Recommendation detail validation" do
      wrap_request do
        variables = {}
        fields_check = fields().dup
        fields_check["{{action}}"] = "SmartMergeAction"

        $merge_ids.each do |key, value|
          graphql url(service: service),
                  format_request(
                      :node,
                      service: service,
                      crn: key ,
                      type: 'ICMRecommendedAction',
                      fields: fields_check
                  ),
                  variables, headers
          expect_success_graphql
          check(:node, service: service)

          $e2e_inp[:SMART_MERGE][:Recommendations].each do |test|
            if ( test['catalystReservationId'] == json_body[:data][:node][:details][:catalystReservation][:reservationId] and
                test['targetReservationId'] == json_body[:data][:node][:details][:targetReservation][:reservationId] )
              puts "Key : #{key}"
              puts "Scenario #{test['scenario']} is in progress"
              expect_json 'data.node.details.catalystReservation',
                          region: test['catalystReservation']['region'],
                          operatingSystem: test['catalystReservation']['operatingSystem'],
                          tenancy: test['catalystReservation']['tenancy'],
                          committedCount: test['catalystReservation']['committedCount'],
                          exchangeCount: test['catalystReservation']['exchangeCount'],
                          instanceType: test['catalystReservation']['instanceType'],
                          normalizationFactor: test['catalystReservation']['normalizationFactor'],
                          discountedPricePerHour: test['catalystReservation']['discountedPricePerHour'],
                          termIncreasedByMonths: test['catalystReservation']['termIncreasedByMonths'],
                          monthlyCommit: test['catalystReservation']['monthlyCommit'],
                          duration: test['catalystReservation']['duration'],
                          instanceSize: test['catalystReservation']['instanceSize'],
                          instanceTypeFamily: test['catalystReservation']['instanceTypeFamily'],
                          instanceTypeId: test['catalystReservation']['instanceTypeId'],
                          paymentOption: test['catalystReservation']['paymentOption'],
                          remainingHours: test['catalystReservation']['remainingHours']

              expect_json 'data.node.details.targetReservation',
                          region: test['targetReservation']['region'],
                          operatingSystem: test['targetReservation']['operatingSystem'],
                          tenancy: test['targetReservation']['tenancy'],
                          committedCount: test['targetReservation']['committedCount'],
                          exchangeCount: test['targetReservation']['exchangeCount'],
                          instanceType: test['targetReservation']['instanceType'],
                          normalizationFactor: test['targetReservation']['normalizationFactor'],
                          discountedPricePerHour: test['targetReservation']['discountedPricePerHour'],
                          termIncreasedByMonths: test['targetReservation']['termIncreasedByMonths'],
                          monthlyCommit: test['targetReservation']['monthlyCommit'],
                          duration: test['targetReservation']['duration'],
                          instanceSize: test['targetReservation']['instanceSize'],
                          instanceTypeFamily: test['targetReservation']['instanceTypeFamily'],
                          instanceTypeId: test['targetReservation']['instanceTypeId'],
                          paymentOption: test['targetReservation']['paymentOption'],
                          remainingHours: test['targetReservation']['remainingHours']

              expect_json 'data.node.details.benefits',
                          exchangeCost: test['benefits']['exchangeCost'],
                          upfrontPayment: test['benefits']['upfrontPayment'],
                          recurringCost: test['benefits']['recurringCost'],
                          recurringCostPerMonth: test['benefits']['recurringCostPerMonth'],
                          totalCost: test['benefits']['totalCost'],
                          lifetimeDiscountSavings: test['benefits']['lifetimeDiscountSavings'],
                          lifeTimeSavings: test['benefits']['lifeTimeSavings']
            end
          end
        end
      end
    end


    it "SmartBoost Recommendation detail validation" do
      wrap_request do
        variables = {}
        fields_check = fields().dup
        fields_check["{{action}}"] = "SmartBoostAction"
        $boost_ids.each do |key, value|
          graphql url(service: service),
                  format_request(
                      :node,
                      service: service,
                      crn: key ,
                      type: 'ICMRecommendedAction',
                      fields: fields_check
                  ),
                  variables, headers
          expect_success_graphql
          check(:node, service: service)

          $e2e_inp[:SMART_BOOST][:Recommendations].each do |test|
            if ( test['catalystReservationId'] == json_body[:data][:node][:details][:catalystReservation][:reservationId] and
                test['targetReservationId'] == json_body[:data][:node][:details][:targetReservation][:reservationId] )
              # Below two lines are to get the actionId to add the value in assertFile
              #puts (json_body[:data][:node][:details][:basicDetails][:actionId])
              puts "Boost Key : #{key}"
              puts "Scenario #{test['scenario']} is in progress"
              expect_json 'data.node.details.catalystReservation',
                          region: test['catalystReservation']['region'],
                          operatingSystem: test['catalystReservation']['operatingSystem'],
                          tenancy: test['catalystReservation']['tenancy'],
                          committedCount: test['catalystReservation']['committedCount'],
                          exchangeCount: test['catalystReservation']['exchangeCount'],
                          instanceType: test['catalystReservation']['instanceType'],
                          normalizationFactor: test['catalystReservation']['normalizationFactor'],
                          discountedPricePerHour: test['catalystReservation']['discountedPricePerHour'],
                          termIncreasedByMonths: test['catalystReservation']['termIncreasedByMonths'],
                          monthlyCommit: test['catalystReservation']['monthlyCommit'],
                          duration: test['catalystReservation']['duration'],
                          instanceSize: test['catalystReservation']['instanceSize'],
                          instanceTypeFamily: test['catalystReservation']['instanceTypeFamily'],
                          instanceTypeId: test['catalystReservation']['instanceTypeId'],
                          paymentOption: test['catalystReservation']['paymentOption'],
                          remainingHours: test['catalystReservation']['remainingHours']

              expect_json 'data.node.details.targetReservation',
                          region: test['targetReservation']['region'],
                          operatingSystem: test['targetReservation']['operatingSystem'],
                          tenancy: test['targetReservation']['tenancy'],
                          committedCount: test['targetReservation']['committedCount'],
                          exchangeCount: test['targetReservation']['exchangeCount'],
                          instanceType: test['targetReservation']['instanceType'],
                          normalizationFactor: test['targetReservation']['normalizationFactor'],
                          discountedPricePerHour: test['targetReservation']['discountedPricePerHour'],
                          termIncreasedByMonths: test['targetReservation']['termIncreasedByMonths'],
                          monthlyCommit: test['targetReservation']['monthlyCommit'],
                          duration: test['targetReservation']['duration'],
                          instanceSize: test['targetReservation']['instanceSize'],
                          instanceTypeFamily: test['targetReservation']['instanceTypeFamily'],
                          instanceTypeId: test['targetReservation']['instanceTypeId'],
                          paymentOption: test['targetReservation']['paymentOption'],
                          remainingHours: test['targetReservation']['remainingHours'],
                          onDemandPricePerHour: test['targetReservation']['onDemandPricePerHour'],
                          onDemandPricePerMonth: test['targetReservation']['onDemandPricePerMonth'],
                          totalOnDemandCost: test['targetReservation']['totalOnDemandCost']

              expect_json 'data.node.details.benefits',
                          exchangeCost: test['benefits']['exchangeCost'],
                          upfrontPayment: test['benefits']['upfrontPayment'],
                          recurringCost: test['benefits']['recurringCost'],
                          recurringCostPerMonth: test['benefits']['recurringCostPerMonth'],
                          totalCost: test['benefits']['totalCost'],
                          #lifetimeDiscountSavings: test['benefits']['lifetimeDiscountSavings'],
                          lifeTimeSavings: test['benefits']['lifeTimeSavings']
           end
          end
        end
      end
    end # End of test case

    it "SmartReuse Recommendation detail validation" do
      wrap_request do
        variables = {}
        fields_check = fields().dup
        fields_check["{{action}}"] = "SmartReuseAction"
        $reuse_ids.each do |key, value|
          graphql url(service: service),
                  format_request(
                      :node,
                      service: service,
                      crn: key ,
                      type: 'ICMRecommendedAction',
                      fields: fields_check
                  ),
                  variables, headers
          expect_success_graphql
          check(:node, service: service)

          $e2e_inp[:SMART_REUSE][:Recommendations].each do |test|
            if ( test['catalystReservationId'] == json_body[:data][:node][:details][:catalystReservation][:reservationId] )
              # Below two lines are to get the actionId to add the value in assertFile
              #puts (json_body[:data][:node][:details][:basicDetails][:actionId])
              #puts test['scenario']
              puts "Reuse Key : #{key}"
              puts "Scenario #{test['scenario']} is in progress"
              expect_json 'data.node.details.catalystReservation',
                          region: test['catalystReservation']['region'],
                          operatingSystem: test['catalystReservation']['operatingSystem'],
                          tenancy: test['catalystReservation']['tenancy'],
                          committedCount: test['catalystReservation']['committedCount'],
                          exchangeCount: test['catalystReservation']['exchangeCount'],
                          instanceType: test['catalystReservation']['instanceType'],
                          normalizationFactor: test['catalystReservation']['normalizationFactor'],
                          discountedPricePerHour: test['catalystReservation']['discountedPricePerHour'],
                          duration: test['catalystReservation']['duration'],
                          instanceSize: test['catalystReservation']['instanceSize'],
                          instanceTypeFamily: test['catalystReservation']['instanceTypeFamily'],
                          paymentOption: test['catalystReservation']['paymentOption'],
                          remainingHours: test['catalystReservation']['remainingHours'],
                          utilization: test['catalystReservation']['utilization']

              expect_json 'data.node.details.targetReservation',
                          region: test['targetReservation']['region'],
                          operatingSystem: test['targetReservation']['operatingSystem'],
                          tenancy: test['targetReservation']['tenancy'],
                          committedCount: test['targetReservation']['committedCount'],
                          #exchangeCount: test['targetReservation']['exchangeCount'],
                          instanceType: test['targetReservation']['instanceType'],
                          normalizationFactor: test['targetReservation']['normalizationFactor'],
                          onDemandPricePerMonth: test['targetReservation']['onDemandPricePerMonth'],
                          #discountedPricePerHour: test['targetReservation']['discountedPricePerHour'],
                          onDemandPricePerHour: test['targetReservation']['onDemandPricePerHour'],
                          totalOnDemandCost: test['targetReservation']['totalOnDemandCost'],
                          instanceSize: test['targetReservation']['instanceSize'],
                          instanceTypeFamily: test['targetReservation']['instanceTypeFamily'],
                          paymentOption: test['targetReservation']['paymentOption'],
                          utilization: test['targetReservation']['utilization']

              expect_json 'data.node.details.benefits',
                          exchangeCost: test['benefits']['exchangeCost'],
                          upfrontPayment: test['benefits']['upfrontPayment'],
                          recurringCost: test['benefits']['recurringCost'],
                          recurringCostPerMonth: test['benefits']['recurringCostPerMonth'],
                          totalCost: test['benefits']['totalCost'],
                          #lifetimeDiscountSavings: test['benefits']['lifetimeDiscountSavings'],
                          lifeTimeSavings: test['benefits']['lifeTimeSavings'],
                          wastageSavingsPerMonth: test['benefits']['wastageSavingsPerMonth'],
                          lifeTimeWastageSavings: test['benefits']['lifeTimeWastageSavings'],
                          lifetimeOnDemandSavings: test['benefits']['lifetimeOnDemandSavings']

            end
          end
        end
      end
    end
  end
end